@@ -22,6 +22,15 @@ class TaskAlreadyExists(PymkError):
         return 'Error: Task name already exists "%s".' % (self.task_name)
 
 
+class RecipeAlreadyExists(PymkError):
+
+    def __init__(self, recipe_name):
+        self.recipe_name = recipe_name
+
+    def __str__(self):
+        return 'Error: Recipe name already exists "%s".' % (self.recipe_name)
+
+
 class NoMkfileFound(PymkError):
     pass
 
@@ -40,13 +49,13 @@ class CommandAborted(PymkError):
     pass
 
 
-class BadTaskName(PymkError):
+class BadTaskPath(PymkError):
 
-    def __init__(self, taskname):
-        self.taskname = taskname
+    def __init__(self, taskpath):
+        self.taskpath = taskpath
 
     def __str__(self):
-        return 'Error: Bad task name: %s' % (self.taskname)
+        return 'Error: Bad task path: %s' % (self.taskpath)
 
 
 class WrongArgumentValue(PymkError):
@@ -81,3 +90,13 @@ class NotADependencyError(PymkError):
 
     def __str__(self):
         return 'Error: Object "%s" of a task "%s" is not a dependency!' % (str(self.dependency.__name__), self.task.__name__)
+
+
+class WrongPymkVersion(PymkError):
+
+    def __init__(self, pymk_version, mkfile_version):
+        self.pymk_version = pymk_version
+        self.mkfile_version = mkfile_version
+
+    def __str__(self):
+        return 'Error: mkfile need greater version (%s) of pymk (%s). Please update.' % (self.mkfile_version, self.pymk_version)
