@@ -6,9 +6,10 @@ import tempfile
 import logging
 from time import sleep
 
-from pymk.script import import_mkfile, run_tasks, TaskMeta
+from pymk.script import import_mkfile, run_tasks, TaskType, init_recipe
 from pymk.extra import touch
 from pymk.task import Task
+from pymk.recipe import RecipeType
 
 
 class ArgsMockup(object):
@@ -31,6 +32,18 @@ class PymkTestCase(unittest.TestCase):
         self._args = ArgsMockup()
         logging.getLogger('pymk').info('=== (%s) %s ===' % (
             self.__class__.__name__, self._testMethodName))
+        self.patchers = {}
+        self.mocks = {}
+
+        self._init_patchers()
+        self._start_patchers()
+
+    def _init_patchers(self):
+        pass
+
+    def _start_patchers(self):
+        for name, patcher in self.patchers.items():
+            self.mocks[name] = patcher.start()
 
     def tearDown(self):
         sys.path.remove(self._actual_path)
@@ -41,6 +54,8 @@ class PymkTestCase(unittest.TestCase):
             del sys.modules['mkfile']
             del self._mkfile
         self._mkfile = None
+        for name, patcher in self.patchers.items():
+            patcher.stop()
 
     def _template(self, name, out_path=None, vars={}):
         template_path = os.path.join(
@@ -52,12 +67,14 @@ class PymkTestCase(unittest.TestCase):
         out_file.close()
 
     def _import_mkfile(self):
-        TaskMeta.init()
+        TaskType.init()
+        RecipeType.recipes = {}
         self._mkfile = import_mkfile()
+        init_recipe('mkfile')
 
     def _pymk(self):
         def reset_all_task_runned_flags():
-            for name, task in TaskMeta.tasks.items():
+            for name, task in TaskType.tasks.items():
                 task._set_runned(False)
         reset_all_task_runned_flags()
         return run_tasks(self._mkfile, self._args)
